@@ -7,7 +7,7 @@ module DBF
     class NameError < StandardError; end
 
     class Base
-      attr_reader :name, :type, :length, :decimal
+      attr_reader :table, :name, :type, :length, :decimal
 
       # Initialize a new DBF::Column
       #
@@ -15,11 +15,22 @@ module DBF
       # @param [String] type
       # @param [Fixnum] length
       # @param [Fixnum] decimal
-      def initialize(name, type, length, decimal, version, encoding=nil)
-        @name, @type, @length, @decimal, @version, @encoding = clean(name), type, length, decimal, version, encoding
+      def initialize(table, name, type, length, decimal)
+        @table = table
+        @name = clean(name)
+        @type = type
+        @length = length
+        @decimal = decimal
+        @version = table.version
+        @encoding = table.encoding
 
-        raise LengthError, "field length must be greater than 0" unless length > 0
-        raise NameError, "column name cannot be empty" if @name.length == 0
+        unless length >= 0
+          raise LengthError, 'field length must be 0 or greater'
+        end
+
+        if @name.empty?
+          raise NameError, 'column name cannot be empty'
+        end
       end
 
       # Cast value to native type
@@ -27,11 +38,13 @@ module DBF
       # @param [String] value
       # @return [Fixnum, Float, Date, DateTime, Boolean, String]
       def type_cast(value)
+        return nil if length == 0
+
         case type
           when 'N' then unpack_number(value)
           when 'I' then unpack_unsigned_long(value)
           when 'F' then value.to_f
-          when 'Y' then unpack_unsigned_long(value) / 10000.0
+          when 'Y' then (unpack_unsigned_long(value) / 10000.0).to_f
           when 'D' then decode_date(value)
           when 'T' then decode_datetime(value)
           when 'L' then boolean(value)
@@ -40,6 +53,9 @@ module DBF
         end
       end
 
+      # Returns true if the column is a memo
+      #
+      # @return [Boolean]
       def memo?
         @memo ||= type == 'M'
       end
@@ -51,16 +67,20 @@ module DBF
         "\"#{underscored_name}\", #{schema_data_type}\n"
       end
 
-      def self.underscore_name(string)
-        string.gsub(/::/, '/').
-          gsub(/([A-Z]+)([A-Z][a-z])/,'\1_\2').
-          gsub(/([a-z\d])([A-Z])/,'\1_\2').
-          tr('-', '_').
-          downcase
-      end
-
+      # Underscored name
+      #
+      # This is the column name converted to underscore format.
+      # For example, MyColumn will be returned as my_column.
+      #
+      # @return [String]
       def underscored_name
-        @underscored_name ||= self.class.underscore_name(name)
+        @underscored_name ||= begin
+          name.gsub(/::/, '/').
+            gsub(/([A-Z]+)([A-Z][a-z])/,'\1_\2').
+            gsub(/([a-z\d])([A-Z])/,'\1_\2').
+            tr('-', '_').
+            downcase
+        end
       end
 
       private
@@ -73,9 +93,11 @@ module DBF
       end
 
       def decode_datetime(value) #nodoc
-        days, milliseconds = value.unpack('l2')
-        seconds = (milliseconds / 1000).to_i
-        DateTime.jd(days, (seconds/3600).to_i, (seconds/60).to_i % 60, seconds % 60) rescue nil
+        days, msecs = value.unpack('l2')
+        secs = (msecs / 1000).to_i
+        DateTime.jd(days, (secs/3600).to_i, (secs/60).to_i % 60, secs % 60)
+      rescue
+        nil
       end
 
       def decode_memo(value) #nodoc
@@ -95,10 +117,10 @@ module DBF
       end
 
       def encode_string(value) #nodoc
-        if @encoding
-          if String.new.respond_to?(:encoding)
-            value.force_encoding(@encoding).encode(Encoding.default_external, :undef => :replace, :invalid => :replace)
-          else
+        if @encoding && table.supports_encoding?
+          if table.supports_string_encoding?
+            value.force_encoding(@encoding).encode(*encoding_args)
+          elsif table.supports_iconv?
             Iconv.conv('UTF-8', @encoding, value)
           end
         else
@@ -106,6 +128,10 @@ module DBF
         end
       end
 
+      def encoding_args #nodoc
+        [Encoding.default_external, {:undef => :replace, :invalid => :replace}]
+      end
+
       def schema_data_type #nodoc
         case type
         when "N", "F"
@@ -134,9 +160,8 @@ module DBF
       end
 
       def clean(value) #nodoc
-        first_null = value.index("\x00")
-        value = value[0, first_null] if first_null
-        value.gsub(/[^\x20-\x7E]/, "")
+        truncated_value = value.strip.partition("\x00").first
+        truncated_value.gsub(/[^\x20-\x7E]/, '')
       end
 
     end
