@@ -10,8 +10,12 @@ module DBF
 
     def initialize(data, set_encoding)
       @data = data
-      @version, @record_count, @header_length, @record_length, @encoding_key = data.unpack("H2 x3 V v2 x17H2")
+      @version, @record_count, @header_length, @record_length, @encoding_key = unpack_header
       @encoding = DBF::ENCODINGS[@encoding_key] if set_encoding
     end
+
+    def unpack_header
+      data.unpack("H2 x3 V v2 x17H2")
+    end
   end
 end
