@@ -15,6 +15,9 @@ module DBF
           memo_string = memo_string[0, memo_size]
         end
         memo_string
+
+      rescue
+        nil
       end
 
       private
