@@ -36,51 +36,66 @@ module DBF
     end
 
     # Reads attributes by column name
+    #
+    # @param [String, Symbol] key
     def [](key)
       key = key.to_s
       if attributes.has_key?(key)
         attributes[key]
-      elsif index = column_names.index(key)
+      elsif index = underscored_column_names.index(key)
         attributes[@columns[index].name]
       end
     end
 
+    # Record attributes
+    #
     # @return [Hash]
     def attributes
       @attributes ||= Hash[@columns.map {|column| [column.name, init_attribute(column)]}]
     end
 
+    # Overrides standard Object.respond_to? to return true if a
+    # matching column name is found.
+    #
+    # @param [String, Symbol] method
+    # @return [Boolean]
     def respond_to?(method, *args)
-      if column_names.include?(method.to_s)
+      if underscored_column_names.include?(method.to_s)
         true
       else
         super
       end
     end
 
-    def method_missing(method, *args)
-      if index = column_names.index(method.to_s)
-        attributes[@columns[index].name]
-      else
-        super
-      end
-    end
-
     private
 
-    def column_names
-      @column_names ||= @columns.map {|column| column.underscored_name}
+    def method_missing(method, *args) #nodoc
+      if index = underscored_column_names.index(method.to_s)
+        attributes[@columns[index].name]
+      else
+        super
+      end
+    end
+
+    def underscored_column_names # nodoc
+      @underscored_column_names ||= @columns.map {|column| column.underscored_name}
     end
 
     def init_attribute(column) #nodoc
-      value = if column.memo?
-        @memo && @memo.get(memo_start_block(column))
-      else
-        unpack_data(column)
-      end
+      value = column.memo? ? memo(column) : unpack_data(column)
       column.type_cast(value)
     end
 
+    def memo(column) #nodoc
+      if @memo
+        @memo.get(memo_start_block(column))
+      else
+        # the memo file is missing, so read ahead to next record and return nil
+        @data.read(column.length)
+        nil
+      end
+    end
+
     def memo_start_block(column) #nodoc
       format = 'V' if %w(30 31).include?(@version)
       unpack_data(column, format).to_i
