@@ -1,5 +1,13 @@
+begin
+  require 'codeclimate-test-reporter'
+  CodeClimate::TestReporter.start
+rescue LoadError
+end
+
 require 'dbf'
+require 'yaml'
 require 'rspec'
+require 'fileutils'
 
 Encoding.default_external = "UTF-8" if defined?(Encoding)
 
@@ -21,3 +29,7 @@ RSpec.configure do |config|
     end
   end
 end
+
+def fixture_path(filename)
+  "#{DB_PATH}/#{filename}"
+end
